###############################################################
#
#   SuPy - Python initialization and utility code
#
###############################################################

import __builtin__
import os, sys
from ruby import RubyError
import supy

debug = False

#def Print(*args):
#	from sys import stdout
#	for arg in args:
#		stdout.write("%s " % arg)
#	stdout.write("\n")
#	stdout.flush()

class Console(object):

	def __init__(self, out):
		self.out = out
	
	def write(self, data):
		self.out.write(data)
	
	def writelines(self, lines):
		for line in lines:
			self.write(line)
	
	def flush(self):
		pass
	
	def close(self):
		pass
	
	def seek(off, whence = None):
		pass
	
	def tell():
		return 0
	
	def truncate(size = None):
		pass
	

def init_module(namespace):
	#Print("Initializing builtins from 0x%x" % id(namespace), namespace)
	def transfer(name):
		value = getattr(namespace, name)
		setattr(__builtin__, name, value)
	setattr(__builtin__, namespace.name(), namespace)
	consts = namespace.constants()
	for name in consts:
		transfer(name)

def init_name(name, value):
	setattr(__builtin__, name, value)

def init_private(name, value):
	setattr(supy, name, value)

def set_stdout(out):
	con = Console(out)
	sys.stdout = con
	sys.stderr = con

def load_scripts():
	print "Loading Python scripts"
	script_dir = Sketchup.find_support_file("plugins/python")
	#print "Script directory =", repr(script_dir)
	if script_dir not in sys.path:
		sys.path.insert(0, script_dir)
	if script_dir:
		for filename in os.listdir(script_dir):
			module_name = None
			if filename.endswith(".py"):
				module_name = filename[:-3]
			else:
				package_name = filename
				filename = os.path.join(filename, "__init__.py")
				if os.path.exists(os.path.join(script_dir, filename)):
					module_name = package_name
			if module_name:
				print "Loading", filename
				__import__(module_name)

def unload_all():
	from os.path import normpath
	prefix1 = normpath(Sketchup.find_support_file("plugins/python")) + os.sep
	prefix2 = normpath(Sketchup.find_support_file("plugins/supy")) + os.sep
	modules = sys.modules
	modnames = modules.keys()
	for modname in modnames:
		if modname <> "supy":
			module = modules[modname]
			#print "Examining", module
			path = getattr(module, '__file__', None)
			if path:
				path = normpath(path)
				if ((path.startswith(prefix1) or path.startswith(prefix2))
					and (path.endswith(".py")
						or path.endswith(".pyc")
						or path.endswith(".pyo"))):
							if debug:
								print "Unloading", modname
							del modules[modname]

def reload_all():
	unload_all()
	load_scripts()

def get_keep():
	import __main__
	keep = getattr(__main__, 'keep', None)
	if not keep:
		keep = {}
		__main__.keep = keep
	return keep

def get_module_keep(modname):
	keep = get_keep()
	modkeep = keep.get(modname)
	if modkeep is None:
		modkeep = {}
		keep[modname] = modkeep
	return modkeep

def get_keep_info():
	frame = sys._getframe(2)
	modglobals = frame.f_globals
	modname = modglobals.get('__name__')
	if not modname:
		raise LookupError(
			"Unable to determine module from which 'keep' was called")
	modkeep = get_module_keep(modname)
	return modglobals, modname, modkeep

def keep(*names):
	modglobals, modname, modkeep = get_keep_info()
	if debug:
		print "Preserving names from", modname
	for name in names:
		if name in modkeep:
			if debug:
				print "   keep --> %r" % name
			modglobals[name] = modkeep[name]
		elif name in modglobals:
			if debug:
				print "   keep <-- %r" % name
			modkeep[name] = modglobals[name]
		else:
			if debug:
				print "   dropping %r" % name
			try:
				del modkeep[name]
			except KeyError:
				pass

def keep_init(name, initializer):
	modglobals, modname, modkeep = get_keep_info()
	if name not in modkeep:
		if debug:
			print "Initializing %s.%s" % (modname, name)
		modkeep[name] = initializer()
	else:
		if debug:
			print "Restoring %s.%s" % (modname, name)
	modglobals[name] = modkeep[name]

def clear_keep(modname, *names):
	modkeep = get_module_keep(modname)
	if names:
		if debug:
			print "Clearing kept names for %s: %s" % (modname, names)
		for name in names:
			try:
				del modkeep[name]
			except KeyError:
				pass
	else:
		print "Clearing keep for %s" % modname
		modkeep.clear()

def to_length(x):
	try:
		return to_l(x)
	except RubyError:
		raise ValueError("Cannot convert string to length")

import __builtin__
__builtin__.keep = keep
__builtin__.keep_init = keep_init
__builtin__.clear_keep = clear_keep
__builtin__.to_length = to_length
